module Formula where

import Maybe

data Formula = Atom Char|
               Impl Formula Formula |
               And Formula Formula |
               SAnd Formula Formula |
	       Or Formula Formula |
	       Falsum
	       deriving (Eq,Ord)


-- isAtomic tests Formulas (and possibly RSeqs and HSeqs) of being atomic
class Atomic a where
	isAtomic :: a -> Bool

instance Atomic Formula where
	isAtomic (Atom _) = True
	isAtomic Falsum = True
	isAtomic _ = False

fmlasAreAtomic :: [Formula] -> Bool
fmlasAreAtomic = and . (map isAtomic) 

extractFmla :: [Formula] -> Maybe (Formula,[Formula])
extractFmla = extractFromList (not.isAtomic)

extractFromList :: (a -> Bool) -> [a] -> Maybe (a,[a]) 
extractFromList f l = efl l
	where 
	efl [] = Nothing
	efl (a:as) 
		| (not $ f a) && (isNothing $ efl as) = Nothing
		| (not $ f a) && (isJust $ efl as) = Just $ (\(x,y) -> (x,a:y)) $ fromJust $ efl as
		| (f a) = Just (a,as)

data State = St [Formula] [Formula]
		deriving Eq

data GTree = GLeaf State |
	     GNode State [(GTree,String)]  
	     deriving Eq

instance Atomic State where
	isAtomic (St y m) = fmlasAreAtomic y && fmlasAreAtomic m

-- a relational sequent
data RSeq = RSeq [Formula] ROp [Formula]
	deriving (Eq)

data ROp = Leq | Lt deriving (Eq)

-- a relational hypersequent
data HSeq = HSeq [RSeq] deriving (Eq)

data Derivation = Der HSeq Label [Derivation] |
		  Leaf HSeq
	  	  deriving (Eq)

type Label = (String,ROp,Char)


-- isAtomic tests Formulas, RSeqs and HSeqs of being atomic
instance Atomic RSeq where
	isAtomic (RSeq lhs _ rhs) = (and $ map isAtomic lhs) &&
	                            (and $ map isAtomic rhs)

instance Atomic HSeq where
	isAtomic (HSeq rseqs) = and $ map isAtomic rseqs



