module FormulaParser (parseFormula) where

import IO
import Formula
import Text.ParserCombinators.Parsec
import Text.ParserCombinators.Parsec.Expr
import System

expr :: Parser Formula
expr = buildExpressionParser table fmla
       <?> "expression"

table = [[op "/\\" And AssocLeft,
          op "&" SAnd AssocLeft, 
          op "|" Or AssocLeft],
	 [op "->" Impl AssocLeft]
        ]
      where
        op s o assoc
           = Infix (do{ string s; return o}) assoc

fmla = do char '('
          x <- expr
	  char ')'
	  return x
       <|> atom
       <?> "simple expression"

atom :: Parser Formula
atom = do a <- letter
          return (Atom a)
       <|> do char '0'
	      return Falsum
       <?> "atom"

preprocInput :: String -> String
preprocInput = filter (/=' ')

parseFormula ::  String -> IO (Maybe Formula)
parseFormula input
        = case (parse expr "" (preprocInput input)) of
	    Left err -> do hPutStr stderr  "parse error at "
                           hPutStr stderr $ show err
			   return Nothing
            Right x -> return (Just x)


