module Main where 

import IO
import Formula
import System
import List
import Maybe
import GilesOutput
import FormulaParser

playGame :: State -> GTree
playGame s@(St y m)
	| isAtomic s = GLeaf s
	| (fmlasAreAtomic y) = youAttack mnaf (St y mfs)
	| otherwise  = iAttack ynaf (St yfs m)
	where (mnaf,mfs) = fromJust $ extractFmla m
	      (ynaf,yfs) = fromJust $ extractFmla y

youAttack :: Formula -> State -> GTree

youAttack f@(And a b) (St y m) = GNode (St y (f:m)) 
                                       [(playGame (St y (a:m)),"You choose "++show a),
                                        (playGame (St y (b:m)),"You choose "++show b)]

youAttack f@(SAnd a b) (St y m) = GNode (St y (f:m))
                                        [(playGame (St y (b:a:m)),"I choose "++show a++", "++show b),
                                         (playGame (St y (Falsum:m)),"I choose "++ show Falsum )]

youAttack f@(Or a b) (St y m)  = GNode (St y (f:m)) 
                                       [(playGame (St y (a:m)),"I choose "++show a),
                                        (playGame (St y (b:m)),"I choose "++show b)]

youAttack f@(Impl a b) (St y m) = GNode (St y (f:m)) 
                                        [(playGame (St y m),"You grant "++show f),
                                         (playGame (St (a:y) (b:m)),"You attack by asserting " ++ show a ++"\\n I defend by asserting " ++ show b)]


iAttack :: Formula -> State -> GTree
iAttack f@(And a b) (St y m) = GNode (St (f:y) m) 
                                     [(playGame (St (a:y) m),"I choose "++show a),
                                      (playGame (St (b:y) m),"I choose "++show b)]
				      
iAttack f@(SAnd a b) (St y m) = GNode (St (f:y) m)
                                        [(playGame (St (b:a:y) m),"You choose "++show a++", "++show b),
                                         (playGame (St (Falsum:y) m),"You choose "++ show Falsum)]

iAttack f@(Or a b) (St y m)  = GNode (St (f:y) m) 
                                     [(playGame (St (a:y) m),"You choose "++show a),
                                      (playGame (St (b:y) m),"You choose "++show b)]
				      
iAttack f@(Impl a b) (St y m) = GNode (St (f:y) m) 
                                      [(playGame (St y m),"I grant "++show f),
                                       (playGame (St (b:y) (a:m)),"I attack by asserting " ++ show a ++"\\n You defend by asserting " ++ show b)]

		
{-
f1 = Impl (Impl (Atom 'a') (Impl (Atom 'b') Falsum)) Falsum
f2 = Impl (Impl (Atom 'a') Falsum) (Atom 'b')
f3 = Impl (And (Atom 'A') (Or (Impl (Atom 'B') Falsum) (Atom 'C'))) (Or (Atom 'A') (Atom 'C')) -}

usage = "Usage: giles <formula>\n" ++
        "eg: giles \"a & (b -> c)\"\n" ++
	"    giles \"(d | a) /\\  d)\"\n" ++
	"    giles \"p -> (q -> (p & q))\"\n"
			

main = do
	s <- getArgs
	if (null s || head s == "") then do
			hPutStr stderr usage 
			exitFailure 
		else do
		      f <- parseFormula $ head s 
		      if (isNothing f) then exitFailure else
		        putStrLn $ show $ playGame (St [] [fromJust f])
