module HSLatexOutput where

import Formula

instance Show Formula  where
	show f 
		| (head $ s) == '(' = tail $ take (length s - 1) s 
		| otherwise = s
		where
		s = show' f
		show' Falsum = "\\perp "
		show' (Atom c) = c:[]
		show' (SAnd a b) = "(" ++ show' a ++ " \\odot " ++ show' b ++ ")"
		show' (And a b) = "(" ++ show' a ++ " \\wedge " ++ show' b ++ ")"
		show' (Or a b) = "(" ++ show' a ++ " \\vee " ++ show' b ++ ")"
		show' (Impl a b) = "(" ++ show' a ++ " \\rightarrow " ++ show' b ++ ")"

instance Show ROp where
	show Leq = "\\leq "
	show Lt = "< "

instance Show Derivation where
	show d = "\\begin{prooftree}\n" ++ show' d ++ "\\end{prooftree}\n"
		where
		show' (Leaf h) = "\\AxiomC{\\scriptsize(\\(Atomic\\))} \\noLine \\UnaryInfC{"++show h++"}\n"
		show' (Der h l ds) = concatMap show' ds ++ "\\RightLabel{\\scriptsize("++label l++")}"++infCmd (length ds) ++ "{" ++ show h ++ "}\n% "++ show h++": "++ concatMap ((++ "   ").show.derFmla) ds ++ "\n\n"
		infCmd 0 = "\\AxiomC"
		infCmd 1 = "\\UnaryInfC"
		infCmd 2 = "\\BinaryInfC"
		label ("sand",rop,lr) = "$\\odot," ++ show rop ++ "," ++ [lr] ++ "$"
		label ("and",rop,lr) = "$\\wedge," ++ show rop ++ "," ++ [lr] ++ "$"
		label ("or",rop,lr) = "$\\vee," ++ show rop ++ "," ++ [lr] ++ "$"
		label ("impl",rop,lr) = "$\\rightarrow," ++ show rop ++ "," ++ [lr] ++ "$"
		derFmla (Der h' _ _) = h'
		derFmla (Leaf h') = h'

conc :: String -> [String] -> String
conc _ [] = ""
conc sep l = drop (length sep) $ concatMap (sep++) l

instance Show HSeq where
	show (HSeq rs) = "$" ++ conc "\\mid " (map show rs) ++ "$"

instance Show RSeq where
	show (RSeq fsl rop fsr) = conc "," (map show fsl) ++ show rop ++ conc "," (map show fsr)

outputPrefix = "\\documentclass[12pt]{article}\n" ++
               "\\usepackage{bussproofs}\n" ++
	       "\\oddsidemargin 0.0in\n" ++
	       "\\evensidemargin 0.0in\n" ++
	       "\\begin{document}"

outputSuffix = "\\end{document}"
 
