module TCGFormula where

import Maybe

data Formula = Atom Char|
               Impl Formula Formula |
               And Formula Formula |
	       Or Formula Formula |
	       Falsum |
	       Verum
	       deriving (Eq,Ord)


-- isAtomic tests Formulas of being atomic
class Atomic a where
	isAtomic :: a -> Bool

instance Atomic Formula where
	isAtomic (Atom _) = True
	isAtomic Falsum = True
	isAtomic Verum = True
	isAtomic _ = False

eFmlasAreAtomic :: [EFormula] -> Bool
eFmlasAreAtomic = and . (map isAtomic) 

fmlasAreAtomic :: [Formula] -> Bool
fmlasAreAtomic = and . (map isAtomic) 

extractLFmlas :: [EFormula] -> [(EFormula,[EFormula])]
extractLFmlas = extractFromList (\(EFmla _ l _) -> (not.isAtomic)l)

extractRFmlas :: [EFormula] -> [(EFormula,[EFormula])]
extractRFmlas = extractFromList (\(EFmla _ _ r) -> (not.isAtomic)r)

getElos :: [EFormula] -> [Elo]
getElos fs = [ Elo f1 f2 | f1@(EFmla _ a1 b1) <- fs, 
	                    f2@(EFmla _ a2 b2) <- fs,
			    isAtomic f1, isAtomic f2,
			    b1 == a2,
			    f1 /= f2]

extractFromList :: Eq a => (a -> Bool) -> [a] -> [(a,[a])] 
extractFromList f l = map (\x -> (x,filter (/= x) l)) $ filter f l
	
instance Atomic EFormula where
	isAtomic (EFmla _ f1 f2) = isAtomic f1 && isAtomic f2 

data EFormula = EFmla ROp Formula Formula deriving Eq

data Attack = AFmla EFormula LR |
	      AElo Elo  deriving Eq

data Elo = Elo EFormula EFormula 

instance Eq Elo where
	(==) (Elo f1 f2) (Elo g1 g2) = ((f1 == g1) && (f2 == g2)) ||
   	                                ((f1 == g2) && (f2 == g1))

data TCGTree = PNode Attack [TCGTree] |
	     PWin |
             ONode [EFormula] [TCGTree] deriving Eq

data LR = L | R deriving (Eq,Show)

data ROp = Leq | Lt deriving Eq

instance Ord TCGTree where
	(<=) a b = depth a <= depth b 

depth :: TCGTree -> Int
depth PWin = 0
depth (PNode _ ts) = 1 + maximum (map depth ts)
depth (ONode _ ts) = 1 + maximum (map depth ts)


-- smallestTree [] = error "smallestTree []"
