module TCGOutput where 

import TCGFormula;


instance Show Formula where
	show Falsum = "\\bot "
	show Verum = "\\top "
	show (And f1 f2) = "(" ++ show f1 ++ "\\wedge " ++ show f2 ++ ")"
	show (Or f1 f2) = "(" ++ show f1 ++ "\\vee " ++ show f2 ++ ")"
	show (Impl f1 f2) = "(" ++ show f1 ++ "\\rightarrow " ++ show f2 ++ ")"
	show (Atom c) = [c]


efs2s :: [EFormula] -> String
efs2s fs = "\\{" ++ (myTail $ foldl (++) "" $ map ("," ++) $  map show fs) ++ "\\}"
	where
	myTail [] = []
	myTail l = tail l

instance Show ROp where
        show Leq = "\\leq "
        show Lt = "<" 

instance Show EFormula where
        show (EFmla rel f1 f2) = show f1 ++ show rel ++ show f2

instance Show Attack where
        show (AFmla (EFmla rel f1 f2) L) = "\\underline{" ++ show f1 ++ "} " ++ show rel ++ show f2
        show (AFmla (EFmla rel f1 f2) R) = show f1 ++ show rel ++ "\\underline{" ++ show f2 ++ "}"
        show (AElo (Elo f1 f2)) = efs2s [f1,f2]

showTree :: Formula -> TCGTree -> String
showTree sf t = "\\documentclass{article}\n\\usepackage{pst-tree}\n\\begin{document}\n$\n" ++ start ++ 
		 "$\n\\end{document}\n"
		where
		start = "\\pstree[levelsep=10ex]{\\TCircle{P}~[tnpos=r]{"++ show sf ++"}}{" ++ t2s t "r" ++ "}"
		root2s r l lr = "\\pstree[levelsep=10ex]{\\TCircle{" ++ r ++ "}~[tnpos="++lr++"]{"++l++"}}"
		t2s (PNode a [t]) lr = root2s "P" (show a) lr ++ "{%\n" ++ t2s t lr ++ "\n}"
		t2s (PNode a [t1,t2]) lr = root2s "P" (show a) lr ++ "{%\n" ++ t2s t1 "l" ++ "\n" ++ t2s t2 "r" ++ "\n}"
		t2s (ONode os [PWin]) lr = root2s "O" (showOsWin os) lr ++ "{}"
		t2s (ONode os [t]) lr = root2s "O" (efs2s os) lr ++ "{%\n" ++ t2s t lr ++ "\n}"
		showOsWin os = "\\{" ++ (myTail $ foldl (++) "" $ map ("," ++) $ map showbox os) ++ "\\}"
			where
			myTail [] = []
			myTail l = tail l
			showbox f@(EFmla Lt _ Falsum) = "\\fbox{$"++show f++"$}"
			showbox f@(EFmla Lt Verum _) = "\\fbox{$"++show f++"$}"
			showbox f@(EFmla Leq Verum Falsum) = "\\fbox{$"++show f++"$}"
			showbox f@(EFmla Lt (Atom a) (Atom b)) 
				| (a == b) = "\\fbox{$"++show f++"$}"
				| otherwise = show f
			showbox f = show f



		
{-
	show (PNode a as) = "PNode("++ show a++", "++"["++(foldl (++) [] (map show as))++ "])"
	show (ONode fs as) = "ONODE("++efs2s fs++", "++"["++ show (head as)++ "])"
	show (PWin) = "PWIN"
-}
	


instance Show Elo where
	show (Elo f1 f2) = show f1 ++ "," ++ show f2 
