module Main where 

import IO
import TCGFormula
import System
import List
import Maybe
import TCGOutput
import TCGParser



findPStrategy :: Formula -> TCGTree
findPStrategy f = ONode [ef] (findPS [] $ [ef])
	where ef = EFmla Lt f Verum

findPS :: [Elo] -> [EFormula] -> [TCGTree]
findPS hist fs
	| isWinningP fs = [PWin]
	| otherwise = map mkNode (wpfs $ tpfs hist fs)

tpfs :: [Elo] -> [EFormula] -> [(Attack,[([EFormula],[TCGTree])])]
tpfs els fs = map (\(a,es,oss) -> (a, map (\x -> (x, findPS es x)) oss)) (pAttacks els fs)

wpfs ::  [(Attack,[([EFormula],[TCGTree])])] -> [(Attack,[([EFormula],[TCGTree])])]
wpfs = filter (all ((/=[]).snd).snd)  


mkNode :: (Attack,[([EFormula],[TCGTree])]) -> TCGTree
mkNode (a,ots) = PNode a (map (\(os,ts) -> ONode os ts) ots)

-- TODO: Think about nub:

myHead [] = []
myHead (a:as) = [a]

pAttacks :: [Elo] -> [EFormula] -> [(Attack,[Elo],[[EFormula]])] 
pAttacks els fs = map attR (nub $ extractRFmlas fs) ++ map attL (nub $ extractLFmlas fs) ++ map attElo (filterUsedElos $ nub $ getElos fs)
	where
	attL (f@(EFmla rel (And a b) c),fs) = (AFmla f L, els, [fs ++ [EFmla rel a c], fs ++ [EFmla rel b c]])
	attL (f@(EFmla rel (Or a b) c),fs) = (AFmla f L, els, [fs ++ [EFmla rel a c, EFmla rel b c]])
	attL (f@(EFmla Lt (Impl a b) c),fs) = (AFmla f L, els, [fs ++ [EFmla Lt b a, EFmla Lt b c]])
	attL (f@(EFmla Leq (Impl a b) c),fs) = (AFmla f L, els, [fs ++ [EFmla Leq Verum c], fs ++ [EFmla Lt b a, EFmla Leq b c]])
	attR (f@(EFmla rel c (And a b)),fs) = (AFmla f R, els, [fs ++ [EFmla rel c a, EFmla rel c b]])
	attR (f@(EFmla rel c (Or a b)),fs) = (AFmla f R, els, [fs ++ [EFmla rel c a], fs ++ [EFmla rel c b]])
	attR (f@(EFmla Lt c (Impl a b)),fs) = (AFmla f R, els, [fs ++ [EFmla Lt c b], fs ++ [EFmla Leq a b, EFmla Lt c Verum]])
	attR (f@(EFmla Leq c (Impl a b)),fs) = (AFmla f R, els, [fs ++ [EFmla Leq a b], fs ++ [EFmla Leq c b]])
	attElo e@(Elo (EFmla Leq a _) (EFmla Leq _ c)) =(AElo e, e:els, [fs ++ [EFmla Leq a c]])
	attElo e@(Elo (EFmla _ a _) (EFmla _ _ c)) = (AElo e, e:els, [fs ++ [EFmla Lt a c]])
	filterUsedElos = filter (not . (`elem` els))


isWinningP :: [EFormula] -> Bool
isWinningP [] = False
isWinningP (ef:efs)
	| (isAtomic ef && iW ef) = True
	| otherwise = isWinningP efs
	where
	iW (EFmla Lt _ Falsum) = True
	iW (EFmla Lt Verum _) = True
	iW (EFmla Leq Verum Falsum) = True
	iW (EFmla Lt (Atom a) (Atom b)) = (a == b) 
        iW _ = False

reducePStrategy :: TCGTree -> TCGTree
reducePStrategy PWin = PWin
reducePStrategy (PNode a ts) = PNode a (map reducePStrategy ts)
reducePStrategy (ONode os ts) = ONode os [minimum $ map reducePStrategy ts]

{-
f1 = Or (Impl (Atom 'A') (Atom 'B')) (Impl (Atom 'B') (Atom 'A'))
f2 = Or (Atom 'A') (Atom 'B')
-}

usage = "Usage: tcgame <formula>\n" ++
        "eg: tcgame \"(a -> b) | (b -> a)\"\n" ++
	"    tcgame \"(a /\\ b) -> (b /\\ a)\"\n"


main = do
        s <- getArgs
        if (null s || head s == "") then do
                        hPutStr stderr usage
                        exitFailure
                else do
                      f <- parseFormula $ head s
                      if (isNothing f) then exitFailure else
			 let
				jf = fromJust f
			 in
                         putStrLn $ showTree jf $ reducePStrategy $ findPStrategy $ jf

